﻿__copyright__ = "Copyright (c) 2016 CollabNet, Inc. All rights reserved."

from app.scripts import BaseInitializationScript

import app.util
import os
import re

class InitializationScript(BaseInitializationScript):

    scriptDescription = "Setup programmatic rabbitmq users"

    def initialize(self):
        app_user                      = self.cfg['RABBITMQ_APP_SERVICES_USER']
        app_password                  = self.cfg['RABBITMQ_APP_SERVICES_PASSWORD']
        app_admin_user                = self.cfg['RABBITMQ_APP_ADMIN_USER']
        app_admin_password            = self.cfg['RABBITMQ_APP_ADMIN_PASSWORD']
        permission_publisher          = self.cfg['RABBITMQ_PERMISSION_PUBLISHER']
        permission_publisher_password = self.cfg['RABBITMQ_PERMISSION_PUBLISHER_PASSWORD']
        app_ctf_user                  = self.cfg['RABBITMQ_APP_CTF_USER']
        app_ctf_password              = self.cfg['RABBITMQ_APP_CTF_PASSWORD']
        default_queues_regex          = "^(%s|%s).*|amq.default|%s" % ('orchestrate', 'eventq', 'event.hub')
        publisher_queues_regex        = "eventq.ctf_user_permissions|event.hub"
        ctf_admin_queue_regex         = ".*"

        self.existing_passwords = self.readExistingPasswords()
        self.existing_users = self.readExistingUsers()

        self.addRabbitmqUser(app_user, app_password)
        self.setRabbitmqUserPermissions(app_user, default_queues_regex)

        self.addRabbitmqUser(permission_publisher, permission_publisher_password)
        self.setRabbitmqUserPermissions(permission_publisher, publisher_queues_regex)

        self.addRabbitmqUser(app_ctf_user, app_ctf_password)
        self.setRabbitmqUserPermissions(app_ctf_user, ctf_admin_queue_regex)

        # Ensure all passwords are up-to-date with respect to current configuration.
        self.changeRabbitmqPassword(app_admin_user, app_admin_password)
        self.changeRabbitmqPassword(app_user, app_password)
        self.changeRabbitmqPassword(permission_publisher, permission_publisher_password)
        self.changeRabbitmqPassword(app_ctf_user, app_ctf_password)

    def addRabbitmqUser(self, user, password):
        if not self.rabbitmqUserExists(user):
            self.log.info("Adding rabbitmq user %s" % user)
            command = "add_user %s \"%s\"" % (user, password)
            self.__runRabbitmqctl(command)
            self.__writePasswordsFile(user, password)
        else:
            self.log.info("Rabbitmq user %s already exists" % user)
            
    def setRabbitmqUserPermissions(self, user, queues_regex):
        command = "set_permissions -p / %s \"%s\" \"%s\" \"%s\"" % (
            user,
            queues_regex,
            queues_regex,
            queues_regex
        )
        self.__runRabbitmqctl(command)

    def changeRabbitmqPassword(self, user, password):
        if user in self.existing_passwords and self.existing_passwords[user] == password:
            self.log.info("Password for user %s seems current; skipping" % user)
        else:
            self.log.info("Updating password for user %s" % user)
            command = "change_password %s '%s'" % (user, password)
            self.__runRabbitmqctl(command)
            self.__writePasswordsFile(user, password)
    
    def rabbitmqUserExists(self, user):
        if user in self.existing_users:
            return True
        return False

    def readExistingPasswords(self):
        password_file = os.path.expanduser(self.cfg['RABBITMQ_PASSWORDS_FILE'])
        passwords = {}
        if os.path.isfile(password_file):
            with open(password_file) as file_data:
                for line in file_data:
                    m = re.match('^\s*(\w+)\s*:\s*(\S*)\s*', line)
                    if m:
                        passwords[m.group(1)] = m.group(2)
        return passwords

    def readExistingUsers(self):
        output = self.__runRabbitmqctl("list_users")
        users = {}
        for line in output.splitlines():
            m = re.match('^(\S+)\s+\[(.*)\]', line)
            if m:
                users[m.group(1)] = m.group(2)
        return users

    def __writePasswordsFile(self, user, password):
        password_file = self.cfg['RABBITMQ_PASSWORDS_FILE']
        tmp_password_file = password_file + '.tmp'
        app.util.runSystem("touch %s" % password_file, self.log)
        app.util.runSystem("sed '/^%s:.*/d' %s > %s" % (
            user, 
            password_file, 
            tmp_password_file), self.log)
        command = "echo -e \"%s: %s\" >> %s" % (
            user, 
            password,
            tmp_password_file
        )
        app.util.runSystem(command, self.log)
        app.util.runSystem("mv %s %s" % (tmp_password_file, password_file), self.log)
        app.util.runSystem("chmod 600 %s" % password_file, self.log)
        
    # NOTE: command needs to be properly quoted before being passed in
    # Run rabbitmqctl command, and return the output
    def __runRabbitmqctl(self, command):
        local_command = "%s %s" % (self.cfg['RABBITMQ_CTL'], command)
        return app.util.executeCommand(local_command)


