from app.scripts import BaseMigrationScript

import os
import app.util
import shutil

class MigrationScript(BaseMigrationScript):

    scriptDescription = "Migrate RabbitMQ Database to setup the data location"

    def migrate(self):

        # Due to Data file location change
        self.relocateDataFiles()


    def createFolderIfNotExisting(self, folder):
        if not os.path.exists(folder):
            try:
                os.makedirs(folder)
            except:
                self.logMesg("warn", "Creating folder %s failed" % folder)


    def relocateDataFiles(self):
        old_rabbitmq_home_dir = '/opt/collabnet/rabbitmq'
        old_rabbitmq_data_dir = os.path.join(old_rabbitmq_home_dir, 'var/lib/rabbitmq/')
        new_rabbitmq_data_dir = self.cfg['RABBITMQ_DATA_DIR']
        new_rabbitmq_home_dir = '/var/lib/rabbitmq'

        if os.path.isdir(old_rabbitmq_data_dir):
            if os.path.isdir(new_rabbitmq_data_dir):
              os.rename(new_rabbitmq_data_dir, "%s.bak" % (new_rabbitmq_data_dir))

            # copy credential file to new location
            rabbitmq_credential_file = os.path.join(old_rabbitmq_home_dir, '.rabbit_passwords')
            if os.path.isfile(rabbitmq_credential_file):
                shutil.copy(rabbitmq_credential_file, new_rabbitmq_home_dir)

            # copy data files to new location
            shutil.move(old_rabbitmq_data_dir, new_rabbitmq_data_dir)

            # Delete (or unlink) the old data
            if os.path.islink(old_rabbitmq_home_dir):
                self.logMesg("info", "Old RabbitMQ dir %s is a symlink; unlinking" % old_rabbitmq_home_dir)
                self.logMesg("info", "Leaving symlink target %s untouched" % os.path.realpath(old_rabbitmq_home_dir))
                os.unlink(old_rabbitmq_home_dir)
            else:
                self.logMesg("info", "Removing old RabbitMQ dir %s" % old_rabbitmq_home_dir)
                shutil.rmtree(old_rabbitmq_home_dir)

        self.createFolderIfNotExisting(new_rabbitmq_data_dir)

        app.util.runSystem("chown -R %s:%s %s" % (self.cfg['RABBITMQ_USER'], self.cfg['RABBITMQ_GROUP'], new_rabbitmq_data_dir), self.log)